<?php
/**
 * File firewall.speedup.php
 * Class SG_SpeedUp
 * 
 * ver.: 1.6.1
 * date: 21 Mar 2025
 */
/* USAGE:

Insert into index.php

on the top:

<?php 
// OPTIMIZATION START - TOP - Added by SpeedUp (21D467F52F85)
$SG_SPEEDUP_CLASS_FILE = dirname(__FILE__)."/webanalyze/firewall/firewall.speedup.php";
if (file_exists($SG_SPEEDUP_CLASS_FILE)) {
    require_once($SG_SPEEDUP_CLASS_FILE);
    $SG_SpeedUp = new SG_SpeedUp();
    $SG_SpeedUp->Start();
} else die("Error! Absent: ".$SG_SPEEDUP_CLASS_FILE);
// OPTIMIZATION END - TOP - Added by SpeedUp
?>


on the bottom:

<?php 
// OPTIMIZATION START - BOTTOM - Added by SpeedUp (555092211981)
if (class_exists("SG_SpeedUp")) $SG_SpeedUp->Finalize();
// OPTIMIZATION END - BOTTOM - Added by SpeedUp
?>




.htaccess


# OPTIMIZATION - Added by SpeedUp (C94FEE55EE47)

# === Set Correct Content-Type for Fonts ===
AddType application/vnd.ms-fontobject .eot
AddType application/x-font-ttf .ttf
AddType application/x-font-opentype .otf
AddType application/x-font-woff .woff
AddType application/font-woff2 .woff2
AddType image/svg+xml .svg

# === Enable Browser Caching with mod_expires ===
<IfModule mod_expires.c>
    ExpiresActive On
    # Default expiration: 1 hour
    ExpiresDefault "access plus 1 hour"
    
    # Static assets: cache for 1 year
    ExpiresByType text/css "access plus 1 year"
    ExpiresByType text/javascript "access plus 1 year"
    ExpiresByType application/javascript "access plus 1 year"
    ExpiresByType application/x-javascript "access plus 1 year"
    ExpiresByType image/gif "access plus 1 year"
    ExpiresByType image/png "access plus 1 year"
    ExpiresByType image/jpeg "access plus 1 year"
    ExpiresByType image/jpg "access plus 1 year"
    ExpiresByType image/jpe "access plus 1 year"
    ExpiresByType image/webp "access plus 1 year"
    ExpiresByType image/svg+xml "access plus 1 year"
    ExpiresByType image/x-icon "access plus 1 year"
    ExpiresByType image/ico "access plus 1 year"
    ExpiresByType application/x-shockwave-flash "access plus 1 year"
    
    # Fonts: cache for 1 year
    ExpiresByType font/truetype "access plus 1 year"
    ExpiresByType application/x-font-ttf "access plus 1 year"
    ExpiresByType font/opentype "access plus 1 year"
    ExpiresByType application/x-font-otf "access plus 1 year"
    ExpiresByType application/font-woff "access plus 1 year"
    ExpiresByType application/font-woff2 "access plus 1 year"
    ExpiresByType application/vnd.ms-fontobject "access plus 1 year"
    
    # Text and HTML: cache for 1 day
    ExpiresByType text/plain "access plus 1 day"
    ExpiresByType text/html "access plus 1 day"
    ExpiresByType application/x-httpd-php-source "access plus 1 day"
</IfModule>

# === Set Cache-Control Headers with mod_headers ===
<IfModule mod_headers.c>
    # Static assets: public cache
    <FilesMatch "\.(ico|jpe?g|png|gif|svg|webp|swf|woff|woff2|eot|ttf|otf)$">
        Header set Cache-Control "public, max-age=31536000, immutable"
    </FilesMatch>
    
    # CSS and JS: public cache
    <FilesMatch "\.(css|js)$">
        Header set Cache-Control "public, max-age=31536000, immutable"
    </FilesMatch>
    
    # HTML and PHP: private cache, must-revalidate
    <FilesMatch "\.(x?html?|php)$">
        Header set Cache-Control "private, must-revalidate, max-age=0"
    </FilesMatch>
    
    # Prevent caching for sensitive files
    <FilesMatch "\.(log|ini|txt)$">
        Header set Cache-Control "no-store, no-cache, must-revalidate, max-age=0"
    </FilesMatch>
</IfModule>

# === Enable Compression with mod_deflate ===
<IfModule mod_deflate.c>
    # Compress HTML, CSS, JavaScript, Text, XML, and fonts
    AddOutputFilterByType DEFLATE text/plain
    AddOutputFilterByType DEFLATE text/html
    AddOutputFilterByType DEFLATE text/xml
    AddOutputFilterByType DEFLATE text/css
    AddOutputFilterByType DEFLATE application/xml
    AddOutputFilterByType DEFLATE application/xhtml+xml
    AddOutputFilterByType DEFLATE application/rss+xml
    AddOutputFilterByType DEFLATE application/javascript
    AddOutputFilterByType DEFLATE application/x-javascript
    AddOutputFilterByType DEFLATE application/font-woff
    AddOutputFilterByType DEFLATE application/font-woff2
    AddOutputFilterByType DEFLATE font/woff
    AddOutputFilterByType DEFLATE font/woff2
    AddOutputFilterByType DEFLATE image/svg+xml

    # Exclude already compressed formats
    SetOutputFilter DEFLATE
    <IfModule mod_mime.c>
        SetEnvIfNoCase Request_URI \.(?:gif|jpe?g|png|webp|woff|woff2)$ no-gzip
    </IfModule>
</IfModule>

# === Disable ETag to Reduce HTTP Overhead ===
<IfModule mod_headers.c>
    Header unset ETag
</IfModule>
FileETag None

# === Enable Keep-Alive for Persistent Connections ===
<IfModule mod_headers.c>
    Header set Connection keep-alive
</IfModule>

# === Prevent MIME Type Sniffing ===
<IfModule mod_headers.c>
    Header set X-Content-Type-Options "nosniff"
</IfModule>

# OPTIMIZATION - END - Added by SpeedUp




*/
class SG_SpeedUp {
    
    var $debug = true; 
    var $enabled = true; 
    var $store_cache = true; 
    
    var $timer_start = 0; 
    
    var $dir_cache = ''; 
    var $url = ''; 
    var $cached_info = ''; 
    var $cached_filename = ''; 
    var $cached_filename_md5 = ''; 
    var $settings_ini = array(); 
    
    public static $default_settings = array(
        'debug' => 0,
        'htmlonly' => 1,
        'exptime' => 0,
        'lazyloading' => 1,
        'skip_http_response_code' => array('404'),
    );


    public function Start()
    {
        $this->timer_start = time();
        
        $baseDir = dirname(__FILE__);
        
        $this->dir_cache = $baseDir.'/'.'speedup_cache';
        
        $file_on = $baseDir.'/firewall.speedup.on';
        $file_off = $baseDir.'/firewall.speedup.off';
        
        if (file_exists($file_off))
        {
            $this->enabled = false;
            unlink($file_off);
            if (file_exists($file_on)) unlink($file_on);
            return;
        }
        
        if (!file_exists($baseDir.'/firewall.speedup.on')) $this->enabled = false;
        
        if (isset( $_GET['nocache'] ) ) $this->enabled = false;
        if (!$this->enabled) return;
        
        // Read firewall.speedup.ini
        $this->settings_ini = $this->ReadSettingsINI($baseDir);
        $this->settings_ini = array_merge(self::$default_settings, $this->settings_ini);
        
        
        // Check for debug mode
        if (isset($this->settings_ini['debug']) && $this->settings_ini['debug'] == 1) $this->debug = true;
        else $this->debug = false;
        
        // Check if debug for specific IP
        if ($this->debug && (isset($this->settings_ini['debug_ip']) && $this->settings_ini['debug_ip'] != ""))
        {
            $this->SaveDebug('Check IP rule: Allowed: '.$this->settings_ini['debug_ip'].', Current: '.self::Get_visitor_IP_address());
            
            if (trim($this->settings_ini['debug_ip']) != self::Get_visitor_IP_address()) 
            {
                // Don't store this session
                $this->store_cache = false;
                $this->enabled = false;
                $this->SaveDebug('Skip IP: '.self::Get_visitor_IP_address());
                return;
            }
        }
        
                
        // Init
        $this->url = $_SERVER['REQUEST_URI'];
        if ($this->url == '/index.php')    
        {
            // Rewrite some URLs
            $this->url = '/';
            
            if ($this->debug) $this->SaveDebug('Rewrite URL /index.php -> /');
        }
        
        // Check for excluded cookies
        if ($this->isExcludedCookieExists())
        {
            
            if ($this->debug) $this->SaveDebug('Exclude cookie found: '.$this->url);
            $this->enabled = false;
            $this->store_cache = false;
            return;
        }

        
        // Check for excluded URLs
        if ($this->isExcludedURL($this->url))
        {
            if ($this->debug) $this->SaveDebug('Exclude URL: '.$this->url);
            $this->enabled = false;
            $this->store_cache = false;
            return;
        }
        
        
        // Exclude some requests and files
        if (strpos($this->url, '/.') === 0)
        {
            if ($this->debug) $this->SaveDebug('Skip URLs with /.: '.$this->url);
            $this->enabled = false;
            $this->store_cache = false;
            return;
        }
        
        if (strpos($this->url, '/?s=') === 0)
        {
            if ($this->debug) $this->SaveDebug('Skip URLs with /?s=: '.$this->url);
            $this->enabled = false;
            $this->store_cache = false;
            return;
        }
        
        $url_4end = substr($this->url, -4);
        $url_3end = substr($this->url, -3);
        if ( $url_4end == '.php' || $url_4end == '.css' || $url_4end == '.csv' || $url_3end == '.js')
        {
            if ($this->debug) $this->SaveDebug('Skip URLs ext: '.$this->url);
            $this->enabled = false;
            $this->store_cache = false;
            return;
        }

        
        $filename_md5 = md5($this->url);
        $this->cached_info = $filename_md5.'_info.dat';
        $this->cached_filename = $filename_md5.'.dat';
        $this->cached_filename_md5 = $filename_md5.'_md5.dat';
        
        if (!file_exists($this->dir_cache))
        {
            mkdir($this->dir_cache);
            
            $fp = fopen($this->dir_cache.'/'.'.htaccess', 'w');
            $line = 'order deny,allow'."\n".'deny from all'."\n";
            fwrite($fp, $line);
            fclose($fp);
        }

        
        ini_set("zlib.output_compression", 1);
        
        if ($_SERVER["REQUEST_METHOD"] == 'POST')
        {
            $this->store_cache = false;
                
            if ($this->debug) $this->SaveDebug('Detected POST method page: '.$this->url);
        }
        
        if ($this->store_cache)
        {
            $cache_file = $this->dir_cache.'/'.$this->cached_filename;
            
            if (file_exists($cache_file))
            {
                if (isset($this->settings_ini['exptime']) && $this->settings_ini['exptime'] > 0)
                {
                    if (filectime($cache_file) + ($this->settings_ini['exptime'] * 60) < time()) $this->UnlinkCachedPage();  // Expired
                }
            }
            
            if (file_exists($cache_file) && ($cache_filesize = filesize($cache_file)) > 0)
            {
            	$fp = fopen($cache_file, "r");
            	$html = fread($fp, $cache_filesize);
            	fclose($fp);
                
                if ($this->debug) $this->SaveDebug('Load cached page: '.$this->cached_filename.', '.$this->url);
                
                if ($this->settings_ini['lazyloading'] == 1) 
                {
                    if ($html != '' && stripos($html, "</html>") !== false) $html = $this->AddImgLazyLoading($html);
                }
            	
            	die($html);
            }
            else {
                if ($this->debug) $this->SaveDebug('Open live page (cache status: '.$this->store_cache.'): '.$this->url);
                
				ob_start(array($this, 'FinalizePreSave'));
            }
        }
        else {
            if ($this->debug) $this->SaveDebug('Open live page (cache status: '.$this->store_cache.'): '.$this->url);
        }

    }


	public function FinalizePreSave($buffer)
	{
		if (strlen($buffer) == 0) return $buffer;
        
        // Check for htmlonly
        if (isset($this->settings_ini['htmlonly']) && $this->settings_ini['htmlonly'] = 1 || !isset($this->settings_ini['htmlonly']))
        {
            if (strpos($buffer, "</html>") === false) // not found
            {
                return $buffer;
            }
        }


        // Check for special content
        if (isset($this->settings_ini['skip_content']) && count($this->settings_ini['skip_content'])) 
		{
			foreach ($this->settings_ini['skip_content'] as $content_pattern)
			{
				if (preg_match($content_pattern, $buffer, $matches)) 
				{
					// Removed already cached page
					if ($this->debug) $this->SaveDebug('SKIP CONTENT found - remove cache file: '.$this->url);
					if (file_exists($this->dir_cache.'/'.$this->cached_filename)) $this->UnlinkCachedPage();
					$this->enabled = false;
					$this->store_cache = false;
					return $buffer;
				}
			}
		}
        
        // Remove content
        if (isset($this->settings_ini['remove_content']) && count($this->settings_ini['remove_content'])) 
		{
			foreach ($this->settings_ini['remove_content'] as $content_to_remove)
			{
			    $buffer = str_replace($content_to_remove, '<!-- Added by SpeedUp '.$content_to_remove.' -->', $buffer);
			}
		}
        
        
        // Check for content md5
        /*
        $current_content_md5 = md5($buffer);
        
        if (!file_exists($this->dir_cache.'/'.$this->cached_filename_md5))
        {
            // Create content md5 file , page is not cached yet
            if ($this->debug) $this->SaveDebug('Save content md5 file: '.$this->cached_filename_md5.' ['.$current_content_md5.']');
            
            $fp = fopen($this->dir_cache.'/'.$this->cached_filename_md5, 'w');
            fwrite($fp, $current_content_md5);
            fclose($fp);
            
			$this->enabled = false;
			$this->store_cache = false;
			return $buffer;
        }
        else {
            // check if content of the page before and content now are the same. to avoid any tokens and etc
            $last_content_md5 = file_get_contents($this->dir_cache.'/'.$this->cached_filename_md5);
            if ($current_content_md5 != $last_content_md5) 
            {
				if ($this->debug) $this->SaveDebug('Dont cache this page. Different content md5 '.$this->url);
				if (file_exists($this->dir_cache.'/'.$this->cached_filename_md5)) $this->UnlinkCachedPage();
				$this->enabled = false;
				$this->store_cache = false;
				return $buffer;
            }
        }
        */
		
        $fp = fopen($this->dir_cache.'/'.$this->cached_filename, 'a');
        fwrite($fp, $buffer."\n".'<!-- Created by SpeedUp '.date("Y-m-d H:i:s").' -->'."\n");
        fclose($fp);
		
		return $buffer;
	}



    public function Finalize()
    {
        if (!$this->store_cache || !$this->enabled) return;
        
        //$html = $this->html_parts.ob_get_contents();
		ob_flush();
        ob_end_clean();
		
		$cached_file = $this->dir_cache.'/'.$this->cached_filename;
		
        $html = '';
        
		if (file_exists($cached_file))
		{
            $filesize = filesize($cached_file);
            
            if ($filesize > 0)
            {
    			$fp = fopen($cached_file, "r");
    			$html = fread($fp, $filesize);
    			fclose($fp);
                
                if (isset($this->settings_ini['exptime']) && $this->settings_ini['exptime'] > 0)
                {
                    if (filectime($cached_file) + ($this->settings_ini['exptime'] * 60) < time()) $this->UnlinkCachedPage();  // Expired
                }
            }
            else $this->UnlinkCachedPage();  // remove file with size 0
        }
        
        
            // Create cache file
            $http_status = http_response_code();
            if ($this->isExcludedHTTPstatus($http_status))
            {
    			if ($html != '') $this->UnlinkCachedPage();	// Delete created cache file
    			
                if ($this->debug) $this->SaveDebug('Skip http status: '.$http_status.', '.$this->url);			
            }
            else {
                if ($this->store_cache)
                {
                    /*$fp = fopen($this->dir_cache.'/'.$this->cached_filename, 'w');
                    fwrite($fp, $html);
                    fclose($fp);*/
                    
                    // Save index
                    $fp = fopen($this->dir_cache.'/'.'_index.log', 'a');
                    $line = $this->cached_filename.','.$this->url."\n";
                    fwrite($fp, $line);
                    fclose($fp);
                    
                    if ($this->debug) $this->SaveDebug('Store page: '.$this->cached_filename.', '.$this->url.', (http status: '.$http_status.')');
                    
                    // Save index
                    $fp = fopen($this->dir_cache.'/'.$this->cached_info, 'w');
                    $info = array(
                        'created' => date("Y-m-d H:i:s"),
                        'file' => $this->cached_filename,
                        'url' => $this->url,
                        'seconds' => time() - $this->timer_start
                    );
                    $line = json_encode($info);
                    fwrite($fp, $line);
                    fclose($fp);
                }
                else if ($this->debug) 
    				{
    					if ($html != '') $this->UnlinkCachedPage();	// Delete created cache file
    					
    					$this->SaveDebug('Skip store page: '.$this->cached_filename.', '.$this->url.', (http status: '.$http_status.')');
    				}
            }
            

        // Check for double </html>
        if ($html != '') 
        {
            $str = str_replace("</html>", "", $html, $count);
            if ($count != 1)
            {
                if ($this->debug) $this->SaveDebug('Found multi </html> '.$this->cached_filename);
                $this->UnlinkCachedPage();	// Delete created cache file
                
                if ($this->settings_ini['lazyloading'] == 1) $html = $this->AddImgLazyLoading($html);
            }
        }

        die($html);
    }
    
    
    public function AddImgLazyLoading($html)
    {
        $dom = new DOMDocument();
        libxml_use_internal_errors(true);
        $dom->loadHTML(mb_convert_encoding($html, 'HTML-ENTITIES', 'UTF-8'));
        libxml_clear_errors();
        
        $images = $dom->getElementsByTagName('img');
        foreach ($images as $img) {
            if (!$img->hasAttribute('loading')) {
                $img->setAttribute('loading', 'lazy');
            }
        }
        
        return $dom->saveHTML();
    }


    public function UnlinkCachedPage()
    {
        // Info
        $file = $this->dir_cache.'/'.$this->cached_info;
        if (file_exists($file)) unlink($file);
        
        // Content
        $file = $this->dir_cache.'/'.$this->cached_filename;
        if (file_exists($file)) unlink($file);
        
        // md5
        $file = $this->dir_cache.'/'.$this->cached_filename_md5;
        if (file_exists($file)) unlink($file);
    }
    
    
    public function SaveLog($logfile, $txt, $error = false)
    {
        $fp = fopen($this->dir_cache.'/'.$logfile, 'a');
        
        $line = date("Y-m-d H:i:s").' '.$this->Get_visitor_IP_address().' '.$txt."\n";
        if ($error) $line = '!!! ERROR !!! '.$line."\n";
        fwrite($fp, $line);
        fclose($fp);
    }
    
    public function SaveDebug($txt, $error = false)
    {
        $this->SaveLog('_debug.log', $txt, $error);
    }
    
    public function SaveErrorLog($txt, $error = false)
    {
        $this->SaveLog('_error.log', $txt, $error);
    }
    
    public function ReadSettingsINI($baseDir = '')
    {
        if ($baseDir == '') $baseDir = dirname(__FILE__);
        
        $ini_file = $baseDir.'/firewall.speedup.ini';
        if (file_exists($ini_file))
        {
            return parse_ini_file($ini_file);
        }
        else return array();
    }
    
    
    public function isExcludedURL($REQUEST_URI)
    {
        if (!isset($this->settings_ini['exclude_urls']) || count($this->settings_ini['exclude_urls']) == 0) return false;  // not excluded
        
        foreach ($this->settings_ini['exclude_urls'] as $rule_url)
        {
            $rule_url_clean = str_replace("*", "", $rule_url);
            if ($rule_url != '' && $rule_url[0] == '*')
            {
                if ($rule_url[strlen($rule_url)-1] == '*')  // e.g. *xxx*
                {
                    if (stripos($REQUEST_URI, $rule_url_clean) !== false)
                    {
                        return true;    // excluded
                    }
                }
                else {
                    $tmp_pos = stripos($REQUEST_URI, $rule_url_clean);
                    if ($tmp_pos !== false && $tmp_pos + strlen($rule_url_clean) == strlen($REQUEST_URI))     // e.g. *xxx
                    {
                        return true;    // excluded
                    }
                }
            }
            else {
                if ($rule_url[strlen($rule_url)-1] == '*')  // e.g. /xxx*
                {
                    $tmp_pos = stripos($REQUEST_URI, $rule_url_clean);
                    if ( $tmp_pos !== false && $tmp_pos == 0)
                    {
                        return true;    // excluded
                    }
                }
                else {
                    if ($rule_url == $REQUEST_URI)  // e.g. /xxx/
                    {
                        return true;    // excluded
                    }
                }
            }
        }
        
        return false;  // not excluded
    }

    
    public function isExcludedHTTPstatus($http_response_code)
    {
        if (!isset($this->settings_ini['skip_http_response_code']) || count($this->settings_ini['skip_http_response_code']) == 0) return false;  // not excluded
        
        foreach ($this->settings_ini['skip_http_response_code'] as $setting_response_code)
        {
            if ($setting_response_code == $http_response_code) return true;
        }
        
        return false;  // not excluded
    }
    
    public function LogHTTPstatus($http_response_code)
    {
        if (!isset($this->settings_ini['log_http_response_code']) || count($this->settings_ini['log_http_response_code']) == 0) return false;  // don't log
        
        foreach ($this->settings_ini['log_http_response_code'] as $setting_response_code)
        {
            if ($setting_response_code == $http_response_code)
            {
                $fp = fopen($this->dir_cache.'/'.'_http_'.intval($http_response_code).'.log', 'a');
                $line = $this->url."\n";
                fwrite($fp, $line);
                fclose($fp);
                
                return true;
            }
        }
        
        return false;  // don't log
    }
    
    
    public function isExcludedCookieExists()
    {
        if (!isset($this->settings_ini['exclude_cookies']) || count($this->settings_ini['exclude_cookies']) == 0) return false;  // no excluded cookies
        
        if (count($_COOKIE))
        {
            foreach ($this->settings_ini['exclude_cookies'] as $setting_excluded_cookie_info)
            {
                $setting_excluded_cookie_array = explode(',', $setting_excluded_cookie_info);
                $setting_excluded_cookie = trim($setting_excluded_cookie_array[0]);
                if (isset($setting_excluded_cookie_array[1])) $setting_excluded_cookie_size = intval($setting_excluded_cookie_array[1]);
                else $setting_excluded_cookie_size = false;
                
                if (substr($setting_excluded_cookie, -1) == '*')
                {
                    $setting_excluded_cookie = substr($setting_excluded_cookie, 0, -1); // without * in the end
                    
                    foreach ($_COOKIE as $cookie_name => $v)
                    {
                        $tmp_i = strpos($cookie_name, $setting_excluded_cookie);
                        if ($tmp_i !== false && $tmp_i == 0)
                        {
                            if ($setting_excluded_cookie_size !== false)
                            {
                                // Check for cookie size
                                if (strlen($v) >= $setting_excluded_cookie_size) 
                                {
                                    return true;  // excluded cookie found with size
                                }
                                else {
                                }
                            }
                            else return true;  // excluded cookie found
                        }
                    }
                }
                else {
                    foreach ($_COOKIE as $cookie_name => $v)
                    {
                        if ($cookie_name == $setting_excluded_cookie) 
                        {
                            if ($setting_excluded_cookie_size !== false)
                            {
                                // Check for cookie size
                                if (strlen($v) >= $setting_excluded_cookie_size) return true;  // excluded cookie found with size
                            }
                            else return true;  // excluded cookie found
                        }
                    }
                }
            }
        }
        
        return false;  // no excluded cookies
    }
    
    public function Get_visitor_IP_address()
    {
        $ip_address = $_SERVER["REMOTE_ADDR"];
        if (isset($_SERVER["HTTP_X_REAL_IP"]) && filter_var($_SERVER["HTTP_X_REAL_IP"], FILTER_VALIDATE_IP)) $ip_address = $_SERVER["HTTP_X_REAL_IP"];
        if (isset($_SERVER["HTTP_X_FORWARDED_FOR"]) && filter_var($_SERVER["HTTP_X_FORWARDED_FOR"], FILTER_VALIDATE_IP)) $ip_address = $_SERVER["HTTP_X_FORWARDED_FOR"];
        if (isset($_SERVER["HTTP_CF_CONNECTING_IP"]) && filter_var($_SERVER["HTTP_CF_CONNECTING_IP"], FILTER_VALIDATE_IP)) $ip_address = $_SERVER["HTTP_CF_CONNECTING_IP"];
        
        return $ip_address;
    }

}